@@ -266,7 +266,6 @@ update_viewport (ChamplainView *view,
   priv->viewport_x = x - priv->anchor_x - priv->viewport_width / 2.0;
   priv->viewport_y = y - priv->anchor_y - priv->viewport_height / 2.0;
 
-
   if (relocate || force_relocate)
     {
       g_signal_handlers_block_by_func (priv->viewport, G_CALLBACK (viewport_pos_changed_cb), view);
@@ -525,7 +524,7 @@ static void
 champlain_view_dispose (GObject *object)
 {
   DEBUG_LOG ()
-
+  
   ChamplainView *view = CHAMPLAIN_VIEW (object);
   ChamplainViewPrivate *priv = view->priv;
 
@@ -589,6 +588,9 @@ _update_idle_cb (ChamplainView *view)
   DEBUG_LOG ()
 
   ChamplainViewPrivate *priv = view->priv;
+  
+  if (!priv->kinetic_scroll)
+    return FALSE;
 
   clutter_actor_set_size (priv->kinetic_scroll,
       priv->viewport_width,
@@ -659,6 +661,8 @@ champlain_view_realize (ClutterActor *actor)
 
   clutter_actor_realize (actor);
 
+  clutter_actor_grab_key_focus (priv->kinetic_scroll);
+
   /* Setup the viewport according to the zoom level */
   /* resize_viewport (view); */
 
@@ -1100,8 +1104,6 @@ champlain_view_init (ChamplainView *view)
   g_signal_connect (priv->kinetic_scroll, "button-press-event",
       G_CALLBACK (kinetic_scroll_button_press_cb), view);
 
-  clutter_stage_set_key_focus (CLUTTER_STAGE (clutter_stage_get_default ()),
-      priv->kinetic_scroll);
   g_signal_connect (priv->kinetic_scroll, "key-press-event",
       G_CALLBACK (kinetic_scroll_key_press_cb), view);
 
@@ -2452,19 +2454,36 @@ show_zoom_actor (ChamplainView *view,
       GList *children, *child;
       gint size;
       gdouble x_coord, y_coord;
-      gint x_first, y_first;
+      gint x_first, y_first, max_x_end, max_y_end;
 
       size = champlain_map_source_get_tile_size (priv->map_source);
 
       x_coord = priv->viewport_x + priv->anchor_x;
       y_coord = priv->viewport_y + priv->anchor_y;
 
+      if (x_coord < 0)
+        x_coord = 0;
+      if (y_coord < 0)
+        y_coord = 0;
+
       x_first = x_coord / size;
       y_first = y_coord / size;
+      
+      max_x_end = champlain_map_source_get_column_count (priv->map_source, priv->zoom_level);
+      max_y_end = champlain_map_source_get_row_count (priv->map_source, priv->zoom_level);
+
+      if (x_first > max_x_end)
+        x_first = max_x_end;
+      if (y_first > max_y_end)
+        y_first = max_y_end;
 
       priv->anim_start_zoom_level = priv->zoom_level;
-      priv->zoom_actor_longitude = champlain_view_x_to_longitude (view, x_first * size - x_coord);
-      priv->zoom_actor_latitude = champlain_view_y_to_latitude (view, y_first * size - y_coord);
+      priv->zoom_actor_longitude = champlain_map_source_get_longitude (priv->map_source,
+        priv->zoom_level,
+        x_first * size);
+      priv->zoom_actor_latitude = champlain_map_source_get_latitude (priv->map_source,
+        priv->zoom_level,
+        y_first * size);
 
       clutter_group_remove_all (CLUTTER_GROUP (priv->zoom_overlay_actor));
       zoom_actor = clutter_group_new ();
@@ -2484,9 +2503,9 @@ show_zoom_actor (ChamplainView *view,
         }
 
       g_list_free (children);
-
-      gdouble deltax = x_first * size - x_coord;
-      gdouble deltay = y_first * size - y_coord;
+      
+      gdouble deltax = x_first * size - priv->viewport_x - priv->anchor_x;
+      gdouble deltay = y_first * size - priv->viewport_y - priv->anchor_y;
       
       clutter_actor_set_position (zoom_actor, deltax, deltay);
       
@@ -2504,6 +2523,8 @@ show_zoom_actor (ChamplainView *view,
     {
       clutter_actor_set_opacity (priv->kinetic_scroll, 0);
       
+      clutter_group_remove_all (CLUTTER_GROUP (priv->zoom_layer));
+
       priv->zoom_animation = clutter_actor_animate (CLUTTER_ACTOR (zoom_actor),
                 CLUTTER_EASE_IN_OUT_QUAD,
                 350,
